@@ -1,10 +1,10 @@
-# @(#)$Id: Storage.pm 285 2011-07-11 12:40:49Z pjf $
+# @(#)$Id: Storage.pm 321 2011-11-30 00:01:49Z pjf $
 
 package File::MealMaster::Storage;
 
 use strict;
 use namespace::clean -except => 'meta';
-use version; our $VERSION = qv( sprintf '0.6.%d', q$Rev: 285 $ =~ /\d+/gmx );
+use version; our $VERSION = qv( sprintf '0.7.%d', q$Rev: 321 $ =~ /\d+/gmx );
 
 use Data::Section -setup;
 use File::DataClass::Constants;
@@ -96,9 +96,8 @@ sub __original_order {
    my ($hash, $lhs, $rhs) = @_;
 
    # New elements will be  added at the end
-   return  1 unless (exists $hash->{ $lhs }->{_order_by});
-   return -1 unless (exists $hash->{ $rhs }->{_order_by});
-
+   exists $hash->{ $lhs }->{_order_by} or return  1;
+   exists $hash->{ $rhs }->{_order_by} or return -1;
    return $hash->{ $lhs }->{_order_by} <=> $hash->{ $rhs }->{_order_by};
 }
 
@@ -212,7 +211,7 @@ File::MealMaster::Storage - MealMaster food recipe file storage
 
 =head1 Version
 
-0.6.$Revision: 285 $
+0.7.$Revision: 321 $
 
 =head1 Synopsis
 
